#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#include <unistd.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <X11/Xlib.h>

struct options {
  int whichled;
  int sleeptime;
  const char *file;
};

struct Xstate {
  Display *display;
};

volatile sig_atomic_t signal_shutdown;

static void default_options(struct options *const o)
{
  bzero(o, sizeof(*o));
}

static int mailP(const struct options *const o)
{
  struct stat statbuf;

  if (stat(o->file, &statbuf) == -1) {
    fprintf(stderr, "stat '%s': ", o->file);
    perror("stat");
    exit(1);
  }

  if (statbuf.st_atime < statbuf.st_mtime)
    return 1;

  return 0;
}

static void led(const struct Xstate *const state, int led, int on_or_off)
{
  XKeyboardControl xctl;

  xctl.led = led;

  if (on_or_off)
    xctl.led_mode = LedModeOn;
  else
    xctl.led_mode = LedModeOff;

  XChangeKeyboardControl(state->display, KBLed|KBLedMode, &xctl);
  XFlush(state->display);
}

static void exit_handler(int sig)
{
  signal_shutdown = sig;
}

static void setsignals(void)
{
  struct sigaction siga;

  signal_shutdown = 0;
  siga.sa_handler = exit_handler;
  sigemptyset(&siga.sa_mask);
  siga.sa_flags = 0;

  sigaction(SIGHUP, &siga, (struct sigaction *)0);
  sigaction(SIGINT, &siga, (struct sigaction *)0);
  sigaction(SIGQUIT, &siga, (struct sigaction *)0);
  sigaction(SIGTERM, &siga, (struct sigaction *)0);
}

static void usage(void)
{
  fprintf(stderr, 
	  "Option          meaning [default]\n"
	  "-f              file to watch [searched for]\n"
	  "-l              which led to use [2]\n"
	  "-s              time to sleep between stats in seconds [3]\n"
	  );
  exit(1);
}

static void setfilename(struct options *const o)
{
  struct stat statbuf;
  static char mailfile[MAXPATHLEN+1];
  char *s;

  if (o->file != NULL)
    return;

  if ( (s = getenv("MAILFILE"))) {
    if (stat(s,&statbuf) < 0) {
      fprintf(stderr, "File1 '%s': ", mailfile);
      perror("stat");
      exit(1);
    }
    strncpy(mailfile, s, sizeof(mailfile));
  }
  else {
    if (! (s = getenv("USER"))) {
      fprintf(stderr, "Can't get either $USER nor $MAILFILE\n");
      exit(1);
    }

    snprintf(mailfile, sizeof(mailfile), "/var/mail/%s", s);
    if (stat(mailfile, &statbuf) == -1) {
      if  (errno != ENOENT) {
	fprintf(stderr, "File2 '%s': ", mailfile);
	perror("stat");
	exit(1);
      }
      /* Didn't exist, next try */
      else {
	snprintf(mailfile, sizeof(mailfile), "/var/spool/mail/%s", s);
	if (stat(mailfile, &statbuf) < 0) {
	  fprintf(stderr, "Can't find a useful mailfile\n");
	  exit(1);
	}
      }
    }
  }

#if 0
  printf("Using file '%s'\n", mailfile);
#endif
  o->file = strdup(mailfile);
}

int main(int argc, char *argv[])
{
  struct Xstate state;
  int ledstate;
  struct options o;
  int ch;

  default_options(&o);

  while ((ch = getopt(argc, argv, "f:l:s:")) != -1) {
    switch (ch) {
    case 'f': o.file = optarg; break;
    case 'l': o.whichled = atoi(optarg); break;
    case 's': o.sleeptime = atoi(optarg); break;
    default: usage();
    }
  }
  argc -= optind;
  argv += optind;
  if (argc != 0)
    usage();

  if (o.whichled < 1)
    o.whichled = 1;
  if (o.sleeptime < 1)
    o.sleeptime = 3;
  setfilename(&o);

  setsignals();

  if ((state.display = XOpenDisplay(NULL)) == NULL) {
    fprintf(stderr,"Can't open display\n");
    exit(1);
  }

  ledstate = 0;
  led(&state, o.whichled, ledstate);

  while (signal_shutdown == 0) {
    if (mailP(&o)) {
      ledstate = 1;
      led(&state, o.whichled, ledstate);
    }
    else {
      if (ledstate != 0) {
	ledstate = 0;
	led(&state, o.whichled, ledstate);
      }
    }
    sleep(o.sleeptime);
  }

  led(&state, o.whichled, 0);
  XCloseDisplay(state.display);

  return 0;
}
